
function [ebeta, AClen, AC, nIter] = pfbswl1(X,y,lambda,W,opts)
% ========================================================================
% PFBS(ISTA) solving 
% min_{beta}{1/2||Xbeta-y||^2 + alpha/2||beta||^2 ||W*beta||_{1}}
% where, W is the diagnal weight matrix ( only input the diagnal vector)
% Copyright (c), Des, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
% ========================================================================
% ========================================================================
% Input:  
% X:            the covariate matrix (nomalized) in R^{n*p}
% y:            response vector in R^{n}
% W:            the weight matrix (default: ones(p,1))
% opts is a structrue contained 
% opts.J:       maxiteration number in SSN for safe guard   (defaut: 5*n)
% opts.mu:      stop parameter when |df|>mu*n  (defaut: 0.5)
% opts.normnX:  step length that make convergence (default norm(nX))
% opts.initial: initial value, (default:zeros(p,1))
% opts.Xty :    X'*y
% opts.stop:    stop when norm(beta^k+1)/norm(beta^k+1-beta^k)<opts.stop 
%               (default,1e-5)
% Output: 
% ebeta:    an estimate of regression coefficient 
% AClen:    acive set size  of ebeta 
% AC:       acive set of ebeta 
% nIter:    the iteration number when CD stop
% ========================================================================
[n,p] = size(X);
if nargin < 3
    disp('error, not enough inputment')
elseif nargin > 5
    disp('error, too much inputment')
elseif nargin == 3 
    W = ones(p,1);
    Xty = X'*y;
    J = 5*n;
    mu = 0.5;
    betainitial = zeros(p,1);
    stoprule = 1e-5;
    alpha = 0;
elseif nargin == 4 || isempty(opts)
    Xty = X'*y;
    J = 5*n;
    mu = 0.5;
    betainitial = zeros(p,1);
    stoprule = 1e-5;
    alpha = 0;
else
    Xty = opts.Xty;
    J = opts.J;
    mu = opts.mu;
    betainitial = opts.initial;
    normnX = opts.normnX;
    stoprule = opts.stop;
    alpha = opts.alpha;
end

nIter = 0;
ebeta = zeros(p,1);
AC = [];
AClen = 0;
stop  = stoprule+1;
mun = mu*n;
Eps = 1e-8;
wac = find(W);
while stop > stoprule && nIter < J && AClen < mun
    nIter = nIter + 1;
    tempbeta = betainitial + (Xty - X'*(X*betainitial)+ alpha*betainitial)/(normnX);
    AC = find(abs(tempbeta)>(lambda/normnX));
    AClen = length(AC);
    ebeta = tempbeta;
    ebeta(wac) = softth (tempbeta(wac),W(wac)/normnX);
    stop = norm(betainitial-ebeta)/(norm(betainitial)+ Eps);
    betainitial = ebeta;
end
%% Subfunction
function thz = softth(z,th)
        thz = max(abs(z)-th,0).*sign(z);
end
end