function d = getdual(x,di,p,opts,A)
%-----------------------------------------------------------------------%
% compute the dual variable for nonconvex penalties on the active set   %
% INPUTS:                                                               %
%     x  --- current primal variable                                    %
%     p  --- current (x + d)                                            %
%   opts --- opts (lam,tau,method)                                      %
%     A  --- active set                                                 %
% OUTPUTS:                                                              %
%     d  --- dual variable on A                                         %
% (c) created by Yuling Jiao (yulingjiaomath@whu.edu.cn)                %
%                 Bangti Jin (bangti.jin@gmail.com)                     %
% Created on March 17, 2013                                             %
%-----------------------------------------------------------------------%
lam = opts.lam;
tau = opts.tau;
method = opts.method;

n = length(A);
d = zeros(n,1);
switch method 
    case 'l0'
        d = zeros(n,1);
    case 'bridge'
        d = zeros(n,1);
        xt= x(A);
        Tstar=(2*lam*(1-tau))^(1/(2-tau));
        ii=find(abs(xt)>=Tstar);
%         ii = find(abs(xt)>=(2*lam*(1-lam))^(1/(2-tau)));
        d(ii) =lam*tau*(abs(xt(ii))).^tau./(xt(ii));
      
%       ii = find(xt);
%       d(ii) =lam*tau*(abs(xt(ii))).^tau./(xt(ii));
%       d(ii) = lam*tau*min(1,abs(xt(ii))).^tau./(abs(xt(ii)));

     
    case 'scad'
        d = zeros(n,1);
        pt = p(A);
        xt = x(A);
        dii= di(A);
        i0=find((xt.*dii)>=0);
        
        jj = intersect(find(abs(pt)<=2*lam),find(abs(pt)>lam));
        ii=intersect(jj,i0);
        d(ii) = lam*sign(pt(ii));
        
        jj = intersect(find(abs(pt)>2*lam),find(abs(pt)< tau*lam)); 
        ii=intersect(jj,i0);
        d(ii) = (tau*lam*sign(pt(ii))-xt(ii))/(tau-1);
    case 'mcp'
        d  = zeros(n,1);
        pt = p(A);
        xt = x(A);
        dii= di(A);
        i0=find((xt.*dii)>=0);
        
        jj=intersect(find(abs(pt)>lam),find(abs(pt)<tau*lam));
        ii=intersect(jj,i0);
        d(ii) = (lam*tau*sign(pt(ii))-xt(ii))/tau;
    case 'cl1'
        d = zeros(n,1);
        pt = p(A);
        ii = intersect(find(abs(pt)>lam),find(abs(pt)<(tau+0.5)*lam));
        d(ii) = lam*sign(pt(ii));
end
end