 function [x,acsize,xhis,nIter] = updasreal(X,y,alpha,lambda,tau,method,initial,maxiter,mu)
% Solving nonconvex regularized 
%      1/2||X*x-y||^2 + alpha/2||x||^2 + P_{lambda,tao}(x)
% with a unified primal-dual active set  algorithm (UPDAS)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input:
% X:           the  matrix in R^{n*p}, unite normalized 
% y:           abservation  vector in R^{n}
% alpha:       is a user given parameter to handle the illposeness of X (default: 0)
% lambda:      also a user given regualrization paramter for sparsity (defaut: ln(n))
% method:      q in [0,1] for l_{q}; 2 for scad; 3 for mcp; 4 for cap-l_1
% tau:         parameter in P_{lambda,tao} controls the concavexity of P. 
%              default: tau  = [] for method = 0 and 1; tau = method for method in
%              (0,1); tau = 3.7 for method = 2; tau = 2.7 for method = 3;
%              tau = 1.5 for method = 4;
% mu;          stop when acsize>mu*n
% maxiter:     max number of iteration
% initial:     initial value
% Output: 
% x:            a sparse solution 
% acsize:       acive set size of x
% xhis:         x in each iteration 
% nIter:        the iteration number when UPDAS stop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% method == 0 for l0
% method in (0,1) for bridge
% method  == 1 for l1 (the only convex cace)
% method  == 2 for scad
% method  == 3 for mcp
% method  == 4 for capped-l1
[n,p] = size(X);
if nargin > 9
    disp('error, too much inputments')
end
if nargin < 9
    mu = 0.5;
end
if nargin < 8
    maxiter = 10;
end
if nargin < 7
    initial = zeros(p,1);
end
if nargin < 6
    method = 0;
end 
if nargin < 5
    tau = [];
end
if nargin < 4
    lambda = log(n);
end
if nargin < 3
    alpha = 0;
end
if nargin < 2
    disp('error, not enough inputment')
end

if method == 0
    T = sqrt(2*lambda);
elseif 0 < method <1
    T = (2-tau)*(2*(1-tau))^((tau-1)/(2-tau))*lambda^(1/(2-tau));
else 
    T = lambda;   
end

S = 1:p;
Xty = X'*y;
pd = initial + Xty - X'*(X*initial);

x = zeros(p,1);
xhis = x;
Ac = find(abs(pd)>T);
acsize = length(Ac);
Ic = S;
Ic(Ac) = [];
stop = 1;
nIter = 0;
while stop ~=0 && nIter < maxiter && acsize<=mu*n
nIter = nIter + 1;
x = zeros(p,1);
d = x;
dualac = getdualac(x,pd,lambda,tau,method,Ac);
d(Ac) = dualac;
Xac = X(:,Ac);
rightac =  (Xty(Ac) - dualac);
G = Xac'*Xac+alpha*eye(acsize);
if method == 1
[L,U] = getcholeskyfac(G);
x(Ac) = U\(L\rightac);
else
    x(Ac) = G\rightac;
end
xhis = [xhis,x];
d(Ic) = Xty(Ic) - X(:,Ic)'*(Xac*x(Ac));
pd = x + d;
oAc = Ac;
oacsize = acsize;
Ac = find(abs(pd)>T);
acsize = length(Ac);
Ic = S;
Ic(Ac) = [];
if acsize == oacsize
    if Ac == oAc
        stop = 0;
    end
end
end
    
