function [X,D,y,ye,xe,A]=gendata(n,p,K,sigma,ratio,seednum,kind,cor)
%========================================================================%
% INPUTS:                                                                %
%     n   ---- number of samples                                         %
%     p   ---- signal length                                             %
%     K   ---- number of nonzero elements in the signal                  %
%  ratio  ---- range of value in the signal (= 10^ratio)                 %
%  sigma  ---- noise variance                                            %
% seednum ---- seed number                                               %
%   kind  ---- type of sample matirx                                     %
%          1 for Random Gaussian,                                        %
%          2 for Heviside matrix,                                        %
%          3 for correlated Random Gaussian                              %
%    cor  ---- corr. coeff. (default 0.2)                                %
% OUTPUTS:                                                               %
%     X   ---- normalized sample matrix                                  %
%     D   ---- diagnal mtrix makes X normalized                          %
%     y   ---- data vector with noise                                    %
%    ye   ---- data vector without noise                                 %
%    xe   ---- true signal                                               %
%     A   ---- the support of xe                                         %
% (c) by Yuling Jiao(yulingjiaomath@whu.edu.cn) and Bangti Jin           %
% Created on Mar, 17, 2013                                               %
%========================================================================%

rand('seed',seednum);   % fix seed
randn('seed', seednum); % fix seed

% generate signal
xe = zeros(p,1);     % true signal  
q = randperm(p);
A = q(1:K);
if ratio ~= 0
    vx = ratio*rand(K,1);
    vx = vx-min(vx);
    vx = vx/max(vx)*ratio;
    xe(A) = 10.^vx.*sign(randn(K,1));
else
    xe(A) = sign(randn(K,1));
end
A = find(xe);
% generate matrix X
switch kind 
    case 1 
        X = randn(n,p);
    case 2
        if n~=p, disp('warning, in heviside case, n should equal to p'); end
        X = tril(ones(p,1)*ones(1,p));
    case 3
        if ~exist('cor','var'), cor = 0.2; end
        X = randn(n,p);
        for j = 2:p-1
            X(:,j) = X(:,j) + cor*X(:,j+1)+ cor*X(:,j-1);
        end
end

% generate right hand side
ye  = X*xe;
y   = ye + sigma*randn(n,1);
[X,D] = normalize(X); % just for simplicity, for your own data you should first 
                  % normized the column  

end