
function [nX,D,y,ye,xtrue,supp]=gendata2(n,p,K,sigma,cor,ratio,seednum,isnorm,kind,s)
%========================================================================
% Input:
% n:           number of  samples 
% p:           length of underlying solution  
% K:           number of non-zero elemet in  underling solution 
% ratio:       range of value in underlying solution (= 10^ratio);
% sigma:       varince of  noise 
% cor:         corelation number of columns in nX 
% seednum:     the seed number 
% isnorm:      unite normlize ther columns of  nX if set to be 1. 
% kind:        generate different sample matirx, 1 for Random Gaussian 
%              2 for Heviside matrix; 
% s:           magnititue for sign signal;
% Output:
% nX:          nomalized sample matrix   if isnormstan = 1
% D:           [], if isnormstan = 0; diagnal matrix if isnormstan = 1
% y:           abservation  vector 
% ye:          abservation vector without noise 
% xtrue:       underlying solution 
% supp:        the support of xtrue 
% Copyright (c), Dec, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
%======================================================================== 

rand('seed',seednum);   % fix seed 
randn('seed', seednum); % fix seed 
ratio = ratio*20;
disp('generate data ...')
t = cputime;
% creat xtrue
xtrue = zeros(p,1);  
q = randperm(p);
supp = q(1:K);
vx = ratio/20*rand(K,1);
vx = vx- min(vx);
vx = vx/max(vx)*ratio/20;
if ratio ~= 0
xtrue(supp) = 10.^vx.*sign(randn(K,1));
else
xtrue(supp) = s*sign(randn(K,1)); % generarate singal with +/-s
end

% creat X 
if kind  == 1  % random Gaussian
X = randn(n,p);
  if cor ~=0
   Sigma = speye(p);
   Sigma = full(Sigma);
   for k = 1:p
      for l = 1:p 
        Sigma(k,l)=cor^(abs(k-l));
      end
   end
   X= X*chol(Sigma);
   end
eta = sigma*randn(n,1);
ye = X*xtrue; 
y =  ye + eta;
elseif kind == 2  % heviside case 
    if n~=p
        disp('error, in heviside  case, n should equal to p')
    else
    X = ones(p,1)*ones(1,p);
    X = tril(X);
    end
    eta = sigma*randn(n,1);
    xtrue = s*xtrue;
    ye = X*xtrue; 
    y =  ye + eta;
else  %  heat kernal 
X = heat(p);
% h = 1/p;
% mesh = 0:h:1;
% xexact(1:p) = s*abs(sin((mesh(p)-mesh(1:n))*pi/(mesh(p)-mesh(1))));
% id = randperm(p);
% id = id(1:K);
% xtrue(id) = s*xexact(id);
id = randperm(p);
X = X(id(1:n),:);
eta = sigma*randn(n,1);
ye = X*xtrue; 
y =  ye + eta;
end

% normalize  
if isnorm ==1
%     y = y - mean(y);
   [nX,D] = normalize(X);
else
    nX = X;
    D = [];
end
supp = find(xtrue);
t = cputime - t;
display(sprintf('data generation is  done using time (second) = %g',t))
end

%% sub function 
function [A,b,x] = heat(n,kappa)
%HEAT Test problem: inverse heat equation.
%  This file is from regularization tool by P. Hansen
% [A,b,x] = heat(n,kappa)
%
% A first kind Volterra integral equation with [0,1] as
% integration interval.  The kernel is K(s,t) = k(s-t) with
%    k(t) = t^(-3/2)/(2*kappa*sqrt(pi))*exp(-1/(4*kappa^2*t)) .
% Here, kappa controls the ill-conditioning of the matrix:
%    kappa = 5 gives a well-conditioned problem
%    kappa = 1 gives an ill-conditioned problem.
% The default is kappa = 1.
%
% An exact soltuion is constructed, and then the right-hand side
% b is produced as b = A*x.

% Reference: A. S. Carasso, "Determining surface temperatures from interior
% observations", SIAM J. Appl. Math. 42 (1982), 558-574.  See also L. Elden,
% "The numerical solution of a non-characteristic Cauchy problem for a
% parabolic equation"; in P. Deuflhand and E. Hairer (Eds.), "Numerical
% Treatment of Inverse Problems in Differential and Integral Equations",
% Birkhauser, 1983.

% Discretization by means of simple quadrature (midpoint rule).

% Per Christian Hansen, IMM, Sep, 13, 2001.

% Set default kappa.
if (nargin==1), kappa = 1; end

% Initialization.
h = 1/n; t = h/2:h:1;
c = h/(2*kappa*sqrt(pi));
d = 1/(4*kappa^2);

% Compute the matrix A.
k = c*t.^(-1.5).*exp(-d./t);
r = zeros(1,length(t)); r(1) = k(1); A = toeplitz(k,r);

% Compute the vectors x and b.
if (nargout>1)
  x = zeros(n,1);
  for i=1:n/2
    ti = i*20/n;
    if (ti < 2)
      x(i) = 0.75*ti^2/4;
    elseif (ti < 3)
      x(i) = 0.75 + (ti-2)*(3-ti);
    else
      x(i) = 0.75*exp(-(ti-3)*2);
    end
  end
  x(n/2+1:n) = zeros(1,n/2);
  b = A*x;
end
end